//
//  AppDelegate.m
//  pushlocationnotify
//
//  Created by iPhone Developer on 2013/11/14.
//  Copyright (c) 2013年 iPhone Developer. All rights reserved.
//

#import "AppDelegate.h"

@implementation AppDelegate
@synthesize deviceToken = _deviceToken;

- (BOOL)application:(UIApplication *)application didFinishLaunchingWithOptions:(NSDictionary *)launchOptions
{
    // Override point for customization after application launch.
    // アプリケーションが停止した状態でRemote Notificationを受信すると通知は一旦iOSに送られ、
    // ダイアログやバナーへと表示された通知がタップされるとアプリケーションが起動され、このメソッドが呼び出されます。
    // ホームアイコンがタップされアプリケーションが起動された場合にもこのメソッドが呼び出されます。
    NSLog(@"didFinishLaunchingWithOptions: %@", launchOptions);
    
#if !TARGET_IPHONE_SIMULATOR
    // Remote Notificationsを登録します
	[[UIApplication sharedApplication]
     registerForRemoteNotificationTypes:(UIRemoteNotificationTypeAlert | UIRemoteNotificationTypeBadge | UIRemoteNotificationTypeSound)];
	
	// アプリケーションアイコンのバッジをクリアします
	application.applicationIconBadgeNumber = 0;
    
    // アプリケーションがRemote Notificationによって表示された通知から起動された場合にはuserInfoにその情報がセットされます
    NSDictionary *userInfo = [launchOptions objectForKey:UIApplicationLaunchOptionsRemoteNotificationKey];
    if ( userInfo ) {
        NSLog(@"userInfo: %@", userInfo);
        if ( userInfo ) {
            NSLog(@"remote notification: %@", [userInfo description]);
            NSDictionary *apsInfo = [userInfo objectForKey:@"aps"];
            
            NSString *alert = [apsInfo objectForKey:@"alert"];
            NSLog(@"Received Push Alert: %@", alert);
            
            NSString *sound = [apsInfo objectForKey:@"sound"];
            NSLog(@"Received Push Sound: %@", sound);
            
            NSString *badge = [apsInfo objectForKey:@"badge"];
            NSLog(@"Received Push Badge: %@", badge);
        }
    }
#endif
    
    return YES;
}
							
- (void)applicationWillResignActive:(UIApplication *)application
{
    // Sent when the application is about to move from active to inactive state. This can occur for certain types of temporary interruptions (such as an incoming phone call or SMS message) or when the user quits the application and it begins the transition to the background state.
    // Use this method to pause ongoing tasks, disable timers, and throttle down OpenGL ES frame rates. Games should use this method to pause the game.
}

- (void)applicationDidEnterBackground:(UIApplication *)application
{
    // Use this method to release shared resources, save user data, invalidate timers, and store enough application state information to restore your application to its current state in case it is terminated later. 
    // If your application supports background execution, this method is called instead of applicationWillTerminate: when the user quits.
}

- (void)applicationWillEnterForeground:(UIApplication *)application
{
    // Called as part of the transition from the background to the inactive state; here you can undo many of the changes made on entering the background.
}

- (void)applicationDidBecomeActive:(UIApplication *)application
{
    // Restart any tasks that were paused (or not yet started) while the application was inactive. If the application was previously in the background, optionally refresh the user interface.
}

- (void)applicationWillTerminate:(UIApplication *)application
{
    // Called when the application is about to terminate. Save data if appropriate. See also applicationDidEnterBackground:.
}

- (void)application:(UIApplication *)application didReceiveRemoteNotification:(NSDictionary *)userInfo {
    NSLog(@"didReceiveRemoteNotification");
#if !TARGET_IPHONE_SIMULATOR
    NSLog(@"userInfo: %@", userInfo);
    if ( userInfo ) {
        NSLog(@"remote notification: %@", [userInfo description]);
        NSDictionary *apsInfo = [userInfo objectForKey:@"aps"];
        
        NSString *alert = [apsInfo objectForKey:@"alert"];
        NSLog(@"Received Push Alert: %@", alert);
        
        NSString *sound = [apsInfo objectForKey:@"sound"];
        NSLog(@"Received Push Sound: %@", sound);
        //AudioServicesPlaySystemSound(kSystemSoundID_Vibrate);
        
        NSString *badge = [apsInfo objectForKey:@"badge"];
        NSLog(@"Received Push Badge: %@", badge);
        application.applicationIconBadgeNumber = [[apsInfo objectForKey:@"badge"] integerValue];
    }
#endif
}

+ (NSString *)stringWithUUID {
    // CFUUIDCreateStringは呼び出しごとに新しいUUIDが生成されてしまうので一度生成したらUserDefaultsに保存しておく
    // ※UserDefaultsに保存するとアプリケーションが再インストールされると値がクリアされてしまうのでキーチェーンなどに保存するのが望ましいかもしれない
    // UserDefaultsから保存済みのUUIDを取得する
    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
    NSString *uuidString = [defaults stringForKey:@"UUID"] ;
    if ( !uuidString ) {
        // 新規にUUIDを生成する
        CFUUIDRef uuidObj = CFUUIDCreate(nil);
        // UUIDを文字列に変換する
        uuidString = (__bridge_transfer NSString *)CFUUIDCreateString(nil, uuidObj);
        CFRelease(uuidObj);
        [defaults setValue:uuidString forKey:@"UUID"];
        [defaults synchronize];
    }
	return uuidString;
}

- (void)application:(UIApplication *)application didRegisterForRemoteNotificationsWithDeviceToken:(NSData *)devToken {
#if !TARGET_IPHONE_SIMULATOR
	// アプリケーションの名前とバージョンを取得する
	NSString *appName = [[[NSBundle mainBundle] infoDictionary] objectForKey:@"CFBundleDisplayName"];
	NSString *appVersion = [[[NSBundle mainBundle] infoDictionary] objectForKey:@"CFBundleVersion"];
	
	// アプリケーションが利用可能な通知の方法を取得する
	NSUInteger rntypes = [[UIApplication sharedApplication] enabledRemoteNotificationTypes];
	BOOL pushBadge = (rntypes & UIRemoteNotificationTypeBadge) ? TRUE : FALSE;
	BOOL pushAlert = (rntypes & UIRemoteNotificationTypeAlert) ? TRUE : FALSE;
	BOOL pushSound = (rntypes & UIRemoteNotificationTypeSound) ? TRUE : FALSE;
	
	// デバイスの情報を取得して必要ならこれもプロバイダに送信する
	NSString *deviceUuid = [AppDelegate stringWithUUID];
	UIDevice *dev = [UIDevice currentDevice];
	NSString *deviceName = dev.name;
	NSString *deviceModel = dev.model;
	NSString *deviceSystemVersion = dev.systemVersion;
    
	// APNSから返されるデバイストークンはキャレット(<と>)で囲われているのでこれを削除する
	self.deviceToken = [[[[devToken description]
                          stringByReplacingOccurrencesOfString:@"<"withString:@""]
                         stringByReplacingOccurrencesOfString:@">" withString:@""]
                        stringByReplacingOccurrencesOfString: @" " withString: @""];
	
    NSLog(@"deviceToken: %@", self.deviceToken);
    
    // プロバイダにアプリケーションの情報を送信
	NSString *requestString = [NSString stringWithFormat:@"app_name=%@&app_version=%@&device_uuid=%@&device_token=%@&device_name=%@&device_model=%@&device_version=%@&push_badge=%d&push_alert=%d&push_sound=%d", appName, appVersion, deviceUuid, self.deviceToken, deviceName, deviceModel, deviceSystemVersion, pushBadge, pushAlert, pushSound];
    NSLog(@"requestString:%@", requestString);
    
	NSString *urlString = [NSString stringWithFormat:@"%@%@", BASE_URL, @"register_device.php"];
    
	NSURL *url = [NSURL URLWithString:urlString];
    NSMutableURLRequest *req = [NSMutableURLRequest requestWithURL:url];
    [req setHTTPMethod:@"POST"];
    NSData *myRequestData = [requestString dataUsingEncoding:NSUTF8StringEncoding];
    [req setValue:@"application/x-www-form-urlencoded" forHTTPHeaderField:@"content-type"];
    [req setHTTPBody: myRequestData];
    
    NSError *error = nil;
    NSURLResponse *res;
    NSData *data = [NSURLConnection sendSynchronousRequest:req returningResponse:&res error:&error];
    NSString *result = [[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding];
	NSLog(@"url: %@", url);
	NSLog(@"result: %@", result);
#endif
}

- (void)application:(UIApplication *)application didFailToRegisterForRemoteNotificationsWithError:(NSError *)error
{
    // APNSへのPush Notificationの登録でエラーが発生した場合このメソッドが呼ばれる
    NSString *description = [error description];
    NSLog(@"Error: %@", description);
}

@end
