//
//  ViewController.m
//  pushlocationnotify
//
//  Created by iPhone Developer on 2013/11/14.
//  Copyright (c) 2013年 iPhone Developer. All rights reserved.
//

#import "AppDelegate.h"
#import "ViewController.h"
#import <MapKit/MapKit.h>
#import "SBJson.h"

@interface ViewController ()
{
    CLLocationManager *locationManager_;
	CLLocation *startLocation_;
    NSURLConnection *connection_;
}

@property (strong, nonatomic) IBOutlet MKMapView *mapView;

@end

@implementation ViewController
@synthesize connection = _connection;
@synthesize asyncData = _asyncData;
@synthesize totalBytes = _totalBytes;
@synthesize loadedBytes = _loadedBytes;

- (void)viewDidLoad
{
    [super viewDidLoad];
	// Do any additional setup after loading the view, typically from a nib.
    startLocation_ = nil;
	locationManager_ = [[CLLocationManager alloc] init];
	[locationManager_ setDelegate:self];
    // 位置情報の取得頻度を設定する(10m移動したら通知)
    [locationManager_ setDistanceFilter:(CLLocationDistance)10];
    // 位置情報の更新を開始する
	[locationManager_ startUpdatingLocation];
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

#pragma mark -
#pragma mark CLLocationManager delegate

// Locationのデータがアップデートした際に呼ばれる
- (void)locationManager:(CLLocationManager *)manager
    didUpdateToLocation:(CLLocation *)newLocation
           fromLocation:(CLLocation *)oldLocation
{
    NSString *locationString = [NSString stringWithFormat:@"%lf,%lf", newLocation.coordinate.latitude, newLocation.coordinate.longitude];
    NSLog(@"locationString: %@", locationString);
    
    // 地図の中心点をGPSから取得された位置に移動
    CLLocationCoordinate2D center = newLocation.coordinate;
    //[_mapView setCenterCoordinate:center animated:YES];
    
    NSString *deviceUUID = [AppDelegate stringWithUUID];
    //AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
    // 自分の位置情報とUUIDをプロバイダに送信
    NSString *params = [NSString stringWithFormat:@"device_uuid=%@&current_location=%@", deviceUUID, locationString];
    NSLog(@"params: %@", params);
    NSString *urlString = [NSString stringWithFormat:@"%@%@", BASE_URL, @"update_location.php"];
    NSURL *url = [NSURL URLWithString:urlString];
    NSMutableURLRequest *req = [NSMutableURLRequest requestWithURL:url];
    [req setHTTPMethod:@"POST"];
    NSData *myRequestData = [params dataUsingEncoding:NSUTF8StringEncoding];
    [req setValue:@"application/x-www-form-urlencoded" forHTTPHeaderField:@"content-type"];
    [req setHTTPBody: myRequestData];
    
    connection_ = [[NSURLConnection alloc] initWithRequest:req delegate:self];
    if ( !connection_ ) {
        UIAlertView *alertView = [[UIAlertView alloc] initWithTitle:NSLocalizedString(@"Error", @"Error") message:NSLocalizedString(@"Error occured when retrieve server information.", @"Error occured when retrieve server information.") delegate:self cancelButtonTitle:NSLocalizedString(@"Cancel", @"Cancel") otherButtonTitles:NSLocalizedString(@"Retry", @"Retry"), nil];
        [alertView show];
    }
}

// Locationのデータの取得に失敗した際に呼ばれる
- (void)locationManager:(CLLocationManager *)manager
       didFailWithError:(NSError *)error
{
    NSLog(@"GPS情報の取得に失敗しました。");
    /*
     UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"GPS Error" message:@"Error" delegate:self cancelButtonTitle:@"OK" otherButtonTitles:nil];
     [alert show];
     [alert release];
     */
}

#pragma mark -
#pragma mark NSURLConnection delegate

// 非同期通信レスポンスが返ってきた
- (void)connection:(NSURLConnection *)connection didReceiveResponse:(NSURLResponse *)response
{
	// データを初期化
	_asyncData = [[NSMutableData alloc] initWithData:0];
    
	// プログレスを初期化
	_totalBytes = [response expectedContentLength];
    _loadedBytes = 0;
    NSLog(@"_totalBytes: %d", _totalBytes);
}

// 非同期通信データ受信中
- (void)connection:(NSURLConnection *)connection didReceiveData:(NSData *)data
{
	// データを追加する
	[_asyncData appendData:data];
	// プログレス更新
	_loadedBytes += [data length];
    NSLog(@"_loadedBytes / _totalBytes: %d / %d", _loadedBytes, _totalBytes);
}

// 非同期通信エラー
- (void)connection:(NSURLConnection *)connection didFailWithError:(NSError *)error
{
	// プログレス停止
    _connection = nil;
	_asyncData = nil;
}

// 非同期通信完了
- (void)connectionDidFinishLoading:(NSURLConnection *)connection {
    // プログレス停止など
    NSString *result = [[NSString alloc] initWithData:_asyncData encoding:NSUTF8StringEncoding];
    NSLog(@"result: %@", result);
    id jresult = [result JSONValue];
    NSLog(@"jresult: %@", jresult);
    NSLog(@"classname: %@", NSStringFromClass([jresult class]));
    if ( [jresult isKindOfClass: [NSDictionary class]] ) {
        NSDictionary *dict = jresult;
        NSInteger errorNo = [[dict objectForKey:@"error_no"] intValue];
        NSString *errorMessage = [dict objectForKey:@"error_msg"];
        NSLog(@"errorNo: %d", errorNo);
        NSLog(@"errorMessage: %@", errorMessage);
    }
    result = nil;
}

@end
