//
//  AppDelegate.m
//  pushnotification
//
//  Created by iPhone Developer on 2013/11/14.
//  Copyright (c) 2013年 iPhone Developer. All rights reserved.
//

#import "AppDelegate.h"

@implementation AppDelegate

- (BOOL)application:(UIApplication *)application didFinishLaunchingWithOptions:(NSDictionary *)launchOptions
{
    // Override point for customization after application launch.
    // アプリケーションが停止した状態でPush Notificationを受信すると通知は一旦iOSに送られ、
    // ダイアログやバナーへと表示された通知がタップされるとアプリケーションが起動され、
    // このメソッドが呼び出されます。
    // ホームアイコンがタップされアプリケーションが起動された場合にもこのメソッドが呼び出されます。
    NSLog(@"didFinishLaunchingWithOptions: %@", launchOptions);
#if !TARGET_IPHONE_SIMULATOR
    // APNSに対してRemote Notification(デバイス)の登録を要求します
    // パラメータとして利用可能な通知のタイプ(アラート、バッジ、サウンド)を指定します
	[[UIApplication sharedApplication]
     registerForRemoteNotificationTypes:(UIRemoteNotificationTypeAlert | UIRemoteNotificationTypeBadge | UIRemoteNotificationTypeSound)];
    
    // アプリケーションがRemote Notificationによって表示された通知から起動された場合にはuserInfoにその情報がセットされます
    NSDictionary *userInfo = [launchOptions objectForKey:UIApplicationLaunchOptionsRemoteNotificationKey];
    if ( userInfo ) {
        NSLog(@"userInfo: %@", userInfo);
        if ( userInfo ) {
            NSLog(@"remote notification: %@", [userInfo description]);
            NSDictionary *apsInfo = [userInfo objectForKey:@"aps"];
            
            NSString *alert = [apsInfo objectForKey:@"alert"];
            NSLog(@"Received Push Alert: %@", alert);
            
            NSString *sound = [apsInfo objectForKey:@"sound"];
            NSLog(@"Received Push Sound: %@", sound);
            
            NSString *badge = [apsInfo objectForKey:@"badge"];
            NSLog(@"Received Push Badge: %@", badge);
        }
    }
#endif
    
    return YES;
}
							
- (void)applicationWillResignActive:(UIApplication *)application
{
    // Sent when the application is about to move from active to inactive state. This can occur for certain types of temporary interruptions (such as an incoming phone call or SMS message) or when the user quits the application and it begins the transition to the background state.
    // Use this method to pause ongoing tasks, disable timers, and throttle down OpenGL ES frame rates. Games should use this method to pause the game.
}

- (void)applicationDidEnterBackground:(UIApplication *)application
{
    // Use this method to release shared resources, save user data, invalidate timers, and store enough application state information to restore your application to its current state in case it is terminated later. 
    // If your application supports background execution, this method is called instead of applicationWillTerminate: when the user quits.
}

- (void)applicationWillEnterForeground:(UIApplication *)application
{
    // Called as part of the transition from the background to the inactive state; here you can undo many of the changes made on entering the background.
}

- (void)applicationDidBecomeActive:(UIApplication *)application
{
    // Restart any tasks that were paused (or not yet started) while the application was inactive. If the application was previously in the background, optionally refresh the user interface.
}

- (void)applicationWillTerminate:(UIApplication *)application
{
    // Called when the application is about to terminate. Save data if appropriate. See also applicationDidEnterBackground:.
}

// Remote Notificationを受信した際に呼び出されるdelegateです
// userInfoにNotificationの情報が格納されています
- (void)application:(UIApplication *)application didReceiveRemoteNotification:(NSDictionary *)userInfo
{
#if !TARGET_IPHONE_SIMULATOR
    NSLog(@"didReceiveRemoteNotification");
    NSLog(@"userInfo: %@", userInfo);
    if ( userInfo ) {
        NSLog(@"remote notification: %@", [userInfo description]);
        NSDictionary *apsInfo = [userInfo objectForKey:@"aps"];
        
        NSString *alert = [apsInfo objectForKey:@"alert"];
        NSLog(@"Received Push Alert: %@", alert);
        
        NSString *sound = [apsInfo objectForKey:@"sound"];
        NSLog(@"Received Push Sound: %@", sound);
        
        NSString *badge = [apsInfo objectForKey:@"badge"];
        NSLog(@"Received Push Badge: %@", badge);
    }
#endif
}

// APNSにRemote Notification(デバイス)が登録された際に呼び出されるdelegateです
// devTokenに割り当てられたデバイストークンが格納されています
- (void)application:(UIApplication *)application didRegisterForRemoteNotificationsWithDeviceToken:(NSData *)devToken
{
    NSLog(@"deviceToken: %@", devToken);
#if !TARGET_IPHONE_SIMULATOR
	self.deviceToken = [[[[devToken
                           description]
                          stringByReplacingOccurrencesOfString:@"<"withString:@""]
                         stringByReplacingOccurrencesOfString:@">" withString:@""]
                        stringByReplacingOccurrencesOfString: @" " withString: @""];
	
    NSLog(@"deviceToken: %@", self.deviceToken);
    // 実際にはこの後にプロバイダに対してデバイストークンを通知する処理を実装する必要があります
#endif
}

// APNSにRemote Notification(デバイス)を登録しようとした際にエラーが発生すると呼び出されるdelegateです
- (void)application:(UIApplication *)application didFailToRegisterForRemoteNotificationsWithError:(NSError *)error
{
    if ( [error code] == 3010 ) {
        NSLog(@"Push notificationはシュミレーターでは動作しません");
    }
    else {
        NSLog(@"didFailToRegisterForRemoteNotificationsWithError: %@", error);
        // エラー処理をここに記述する
    }
}

@end
