<?php

$E = error_reporting();
if(($E & E_STRICT) == E_STRICT) error_reporting($E ^ E_STRICT);

require_once 'configure.php';
require_once 'MDB2.php';
require_once 'Log.php';

// Using Autoload all classes are loaded on-demand
require_once 'ApnsPHP/Autoload.php';

// デバッグ用にログファイルに情報を出力しています
$logfile = &Log::factory('file', 'debug.log', 'DEBUG');

$ldb =& MDB2::factory(DATABASE, array('debug' => 2, 'result_buffering' => false));

// Instanciate a new ApnsPHP_Push object
$server = new ApnsPHP_Push_Server(
    ApnsPHP_Abstract::ENVIRONMENT_SANDBOX,
    'apns_development_key_and_cert.pem'
);

// Set the Root Certificate Autority to verify the Apple remote peer
$server->setRootCertificationAuthority('entrust_2048_ca.cer');

// Set the number of concurrent processes
$server->setProcesses(4);

// Starts the server forking the new processes
$server->start();

// Main loop...
//$i = 1;
while ( $server->run() ) {

    // このサンプルではエラー処理は行なっていない
    // Check the error queue
    //$aErrorQueue = $server->getErrors();
    //if (!empty($aErrorQueue)) {
    // Do somethings with this error messages...
    //var_dump($aErrorQueue);
    //}

    // device_infoテーブルから登録済みの全ての登録済みデバイス情報を取得する
    try {
        $params1 = array();
        $sql1 = 'SELECT seq_id';
        $sql1 .= ', Y(current_location) AS latitude, X(current_location) AS longitude';
        $sql1 .= ', device_token FROM device_info';
        $stmt1 = $ldb->prepare($sql1);
        if ( PEAR::isError($stmt1) ) {
        throw new exception($stmt1->getMessage(), EC_DATABASE_ERROR);
        }

        $rs1 = $stmt1->execute();
        if ( PEAR::isError($rs1) ) {
        throw new exception($rs1->getMessage(), EC_DATABASE_ERROR);
        }
    }
    catch ( Exception $e ) {
        $logfile->log('Error:' . $e->getMessage());
    }

    // 取得した情報を配列に格納する
    $results = array();
    while ( $row1 =& $rs1->fetchRow(MDB2_FETCHMODE_OBJECT) ) {
        $results[] = $row1;
    }

    $rs1->free();
    $stmt1->free();

    // 前回のループ以降(updated_on > last_update)情報が更新された自分以外ののiOSデバイスの数を取得する
    $sql2 = 'SELECT COUNT(*) AS updated_count';
    $sql2 .= ' FROM device_info WHERE seq_id != :seq_id AND updated_on < :last_update';
    $sql2 .= " AND MBRContains(GeomFromText(CONCAT('LineString(', :beg_longitude, ' ', :beg_latitude, ', ', :end_longitude, ' ', :end_latitude, ')')), current_location)";
    $stmt2 = $ldb->prepare($sql2);
    if ( PEAR::isError($stmt2) ) {
        throw new exception($stmt2->getMessage(), EC_DATABASE_ERROR);
    }

    $results2 = array();
    foreach ( $results as $row ) {
        try {

            // MkMapViewの範囲指定は(MkCoordinateSpan)に合わせlatitude_delta, longitude_deltaで指定する
            // latitude_delta, longitude_deltaは1度でおおよそ111kmとなる
            $latitude_delta = 0.05;
            $longitude_delta = 0.05;
            $params2 = array();

            // 地図の範囲で絞り込みを行なう
            // latitude_delta, longitude_deltaがそれぞれ0.05なので約5km四方がその対象となる
            $params2['beg_latitude'] = $row->latitude - ($latitude_delta / 2.0);
            $params2['beg_longitude'] = $row->longitude - ($longitude_delta / 2.0);
            $params2['end_latitude'] = $row->latitude + ($latitude_delta / 2.0);
            $params2['end_longitude'] = $row->longitude + ($longitude_delta / 2.0);
            $params2['latitude'] = $row->latitude;
            $params2['longitude'] = $row->longitude;
            $params2['seq_id'] = $row->seq_id;
            $params2['last_update'] = date('Y-m-d H:i:s');

            // SQLステートメントを実行し結果を取得する
            $rs2 = $stmt2->execute($params2);
            if ( PEAR::isError($rs2) ) {
                throw new exception($rs2->getMessage(), EC_DATABASE_ERROR);
            }
        }
        catch ( Exception $e ) {
            $logfile->log('Error:' . $e->getMessage());
        }
        if ( $row2 =& $rs2->fetchRow(MDB2_FETCHMODE_OBJECT) ) {
            // $updated_countに条件に該当するiOSデバイスの数をセットする
            $updated_count = $row2->updated_count + 0;
            print "device_token: ".$row->device_token."\n";
            print "updated_count: ".$updated_count."\n";
            if ( $updated_count > 0 ) {
                // 該当するiOSデバイスが存在すれば、その数をPush NotificationによってiOSデバイスに通知する
                if ( isset($row->device_token) ) {
                    $message = new ApnsPHP_Message($row->device_token);
                    // バッジアイコンに当するiOSデバイスの数を表示する
                    $message->setBadge($updated_count);

                    // 通知センターやダイアログのメッセージに該当するiOSデバイスの数を通知する
                    $message->setText($updated_count . '台の端末が近くにあります');

                    // 通知と同時にデフォルトのサウンドを再生する
                    $message->setSound();

                    // メッセージ(ペイロード)をキューへと登録する
                    $server->add($message);
                }
            }
        }
    }

    // Sleep a little...
    usleep(200000);
}
