<?php

$E = error_reporting();
if(($E & E_STRICT) == E_STRICT) error_reporting($E ^ E_STRICT);

require_once 'configure.php';
require_once 'MDB2.php';
require_once 'Log.php';

// FUNCTION:
//   デバイスからデバイスの現在位置情報を受け取りテーブルを更新します
//
// IN:(POST)
//   device_uuid : デバイスユニークID
//   current_location : デバイスの現在位置("緯度,経度"の文字列)
//
// OUT:(json)
//   error_no : エラーコード
//   error_msg : エラーメッセージ
//
function update_location() {
    try {

        $logfile = &Log::factory('file', 'debug.log', 'DEBUG');

        $device_uuid = $_REQUEST['device_uuid'];
        $current_location = $_REQUEST['current_location'];

        $logfile->log('update_location()');
        $logfile->log('device_uuid:'.$device_uuid);
        $logfile->log('current_location:'.$current_location);

        $ldb =& MDB2::factory(DATABASE, array('debug' => 2, 'result_buffering' => false));

        $params1 = array();
        $sql1 = 'SELECT seq_id FROM device_info WHERE device_uuid=:device_uuid';
        $stmt1 = $ldb->prepare($sql1);
        if ( PEAR::isError($stmt1) ) {
            throw new exception($stmt1->getMessage(), EC_DATABASE_ERROR);
        }

        $params1['device_uuid'] = $device_uuid;

        $rs1 = $stmt1->execute($params1);
        if ( PEAR::isError($rs1) ) {
            throw new exception($rs1->getMessage(), EC_DATABASE_ERROR);
        }

        $do_update = 0;
        if ( $row1 = $rs1->fetchRow(MDB2_FETCHMODE_OBJECT) ) {
            $do_update = 1; // UPDATE
            $seq_id = $row1->seq_id;
        }
        $rs1->free();
        $stmt1->free();
        $logfile->log('seq_id:'.$seq_id);

        $params2 = array();
        if ( $do_update ) {
            list($longitude, $latitude) = explode(',', $current_location);
            $sql2 = 'UPDATE device_info SET ';
            $sql2 .= 'current_location = ';
            $sql2 .= "GeomFromText(CONCAT('POINT(', :latitude, ' ', :longitude, ')')), ";
            $sql2 .= 'updated_on = :updated_on ';
            $sql2 .= 'WHERE seq_id=:seq_id';

            $params2['longitude'] = $longitude;
            $params2['latitude'] = $latitude;
            $params2['updated_on'] = date('Y-m-d H:i:s');
            $params2['seq_id'] = $seq_id;

            $logfile->log('sql: ' . $sql2);
            $stmt2 = $ldb->prepare($sql2);
            if ( PEAR::isError($stmt2) ) {
                throw new exception($stmt2->getMessage(), EC_DATABASE_ERROR);
            }

            $rs2 = $stmt2->execute($params2);
            if ( PEAR::isError($rs2) ) {
                throw new exception($rs2->getMessage(), EC_DATABASE_ERROR);
            }
            //$ldb->commit();
            $rs2->free();
            $stmt2->free();
        }

        $response = array(
            "error_no" => 0,
            "error_msg"  => "No Error."
        );

        $logfile->log('No Error');
        $encode = json_encode($response);
        $logfile->log('encode:'.$encode);
        header("Content-Type: text/javascript; charset=utf-8");
        echo $encode;
    }
    catch ( Exception $e ) {
        $logfile->log('Error:' . $e->getMessage());
        $response = array(
            "error_no" => $e->getCode(),
            "error_msg"  => $e->getMessage()
        );
        $encode = json_encode($response);
        header("Content-Type: text/javascript; charset=utf-8");
        echo $encode;
    }
}

update_location();

?>
